const userAgent = window.navigator.userAgent
// const isAndroid = userAgent.indexOf('Android') !== -1;
const isIOS = (userAgent.indexOf('iPhone') !== -1 || userAgent.indexOf('iPad') !== -1) && !window.isIDE
const isWin = userAgent.indexOf('Windows') !== -1 && !window.isIDE
const isWeb = window.self !== window.top

// invoke callbacks
const callbacks = {}
let callbackIndex = 0

// subscribe handlers
const handlers = {}
const eventPrefix = 'custom_event_'

const invokeHandler = function(event, paramsString, callbackId) {
  if (isIOS) {
    window.webkit.messageHandlers.webInvokeHandler.postMessage({
      C: event,
      paramsString: paramsString,
      callbackId: callbackId,
    })
  } else if (isWin) {
    const payload = {
      C: event,
      paramsString: paramsString,
      callbackId: callbackId,
      type: 'invoke',
    }
    window.FinClipExecuteCommandAsync({
      request: JSON.stringify(payload),
      persistent: false,
      onSuccess: function() { },
      onFailure: function() { },
    })
  } else if (isWeb) {
    const payload = {
      C: event,
      paramsString: paramsString,
      callbackId: callbackId,
      from: 'webview',
      type: 'invoke',
    }
    if (!window.parent) return
    let jsCoreHandleResult = window.parent.postMessage(payload, '*')
    if (typeof jsCoreHandleResult !== 'undefined' && typeof callbacks[callbackId] === 'function' && jsCoreHandleResult !== '') {
      try {
        jsCoreHandleResult = JSON.parse(jsCoreHandleResult)
      } catch (e) {
        jsCoreHandleResult = {}
      }
      callbacks[callbackId](jsCoreHandleResult)
      delete callbacks[callbackId]
    }
  } else {
    let jsCoreHandleResult = FinChatJSCore.webInvokeHandler(event, paramsString, callbackId)
    if (typeof jsCoreHandleResult !== 'undefined' && typeof callbacks[callbackId] === 'function' && jsCoreHandleResult !== '') {
      try {
        jsCoreHandleResult = JSON.parse(jsCoreHandleResult)
      } catch (e) {
        jsCoreHandleResult = {}
      }
      callbacks[callbackId](jsCoreHandleResult)
      delete callbacks[callbackId]
    }
  }
}

const invoke = function(event, params, callback) {
  // postMessage
  const paramsString = JSON.stringify(params || {})
  const callbackId = ++callbackIndex
  callbacks[callbackId] = callback
  invokeHandler(event, paramsString, callbackId)
}
const webInvokeCallbackHandler = function(callbackId, params) {
  const callback = callbacks[callbackId]
  for (let i = 0; i < window.frames.length; i++) {
    window.frames[i].postMessage(JSON.stringify({ callbackId, params }), '*')
  }
  if (typeof callback === 'function') callback(params)
  delete callbacks[callbackId]
}

const webSubscribe = function(eventName, handler) {
  handlers[eventName] = handler
}

const subscribe = function(eventName, handler) {
  handlers[eventPrefix + eventName] = handler
}

const subscribeHandler = function(eventName, data, webviewId, reportParams) {
  // 执行注册的回调
  console.log('subscribeHandler', eventName)
  const handler = handlers[eventName]
  if (typeof handler === 'function') {
    return handler(data, webviewId, reportParams)
  }
}

const publish = function(eventName, params, webviewIds) {
  console.log(eventName, params, webviewIds)
  // publishHandler
  webviewIds = webviewIds || []
  let paramsString = ''
  const event = eventName
  paramsString = JSON.stringify(params)
  webviewIds = JSON.stringify(webviewIds)
  publishHandler(event, paramsString, webviewIds)
}

const publishHandler = function(event, paramsString, webviewIds) {
  // publishHandler
  if (isIOS) {
    window.webkit.messageHandlers.publishHandler.postMessage({
      event: event,
      paramsString: paramsString,
      webviewIds: webviewIds,
    })
  } else if (isWin) {
    const payload = {
      event: event,
      paramsString: paramsString,
      webviewIds: webviewIds,
      type: 'publish',
    }
    window.FinClipExecuteCommandAsync({
      request: JSON.stringify(payload),
      persistent: false,
      onSuccess: function(res) {
        // _this.callbacks[callbackId](res)
        // delete _this.callbacks[callbackId]
      },
      onFailure: function(code, msg) {

      },
    })
    // window.chrome.webview.postMessage({
    //   event: event,
    //   paramsString: paramsString,
    //   webviewIds: webviewIds,
    //   type: 'publish',
    // });
  } else if (isWeb) {
    const payload = {
      event: event,
      paramsString: paramsString,
      from: 'webview',
      type: 'publish',
    }
    if (window.parent) {
      window.parent.postMessage(payload, '*')
    }
  } else {
    FinChatJSCore.publishHandler(event, paramsString, webviewIds)
  }
}

const invokeMethod = function(apiName, opts, innerCb) {
  const options = opts || {}
  const innerFns = innerCb || {}
  const params = {}

  for (const i in options) {
    if (typeof options[i] === 'function') {
      params[i] = options[i]
      delete options[i]
    }
  }
  const sysEventFns = {}
  for (const s in innerFns) {
    if (typeof innerFns[s] === 'function') {
      sysEventFns[s] = innerFns[s]
    }
  }

  invoke(apiName, options, function(res) {
    res.errMsg = res.errMsg || apiName + ':ok'
    const isOk = res.errMsg.indexOf(apiName + ':ok') === 0
    const isCancel = res.errMsg.indexOf(apiName + ':cancel') === 0
    const isFail = res.errMsg.indexOf(apiName + ':fail') === 0
    if (isOk) {
      typeof sysEventFns.beforeAll === 'function' && sysEventFns.beforeAll(res)
      typeof sysEventFns.beforeSuccess === 'function' && sysEventFns.beforeSuccess(res)
      typeof params.success === 'function' && params.success(res)
      typeof sysEventFns.afterSuccess === 'function' && sysEventFns.afterSuccess(res)
    } else if (isCancel) {
      res.errMsg = res.errMsg.replace(apiName + ':cancel', apiName + ':fail cancel')
      typeof params.fail === 'function' && params.fail(res)
      typeof sysEventFns.beforeCancel === 'function' && sysEventFns.beforeCancel(res)
      typeof params.cancel === 'function' && params.cancel(res)
      typeof sysEventFns.afterCancel === 'function' && sysEventFns.afterCancel(res)
    } else if (isFail) {
      typeof sysEventFns.beforeFail === 'function' && sysEventFns.beforeFail(res)
      typeof params.fail === 'function' && params.fail(res)
      typeof sysEventFns.afterFail === 'function' && sysEventFns.afterFail(res)
    }
    typeof params.complete === 'function' && params.complete(res)
    typeof sysEventFns.afterAll === 'function' && sysEventFns.afterAll(res)
  },
  )
}

if (isWeb) {
  window.addEventListener('message', function({ data = {} }) {
    const { type } = data
    if (type === 'webInvokeCallbackHandler') {
      const { callbackId, params } = data
      webInvokeCallbackHandler(callbackId, params)
    } else if (type === 'subscribeHandler') {
      const { eventName, params } = data
      subscribeHandler(eventName, params)
    }
  })
}

window.FinChatJSBridge = {
  invoke,
  invokeCallbackHandler: webInvokeCallbackHandler,
  webInvokeCallbackHandler,
  subscribe,
  subscribeHandler,
  // Mark: 兼容的调用方式
  webSubscribeCallBackHandler: subscribeHandler,
  publish,
  publishHandler,
  webSubscribe,
}
module.exports = {
  invoke,
  invokeCallbackHandler: webInvokeCallbackHandler,
  webInvokeCallbackHandler,
  subscribe,
  subscribeHandler,
  // Mark: 兼容的调用方式
  webSubscribeCallBackHandler: subscribeHandler,
  publish,
  publishHandler,
  invokeMethod: invokeMethod,
  webSubscribe,
}
